#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: reload_httpd.pl

# signal apache to reload it's configuration files.

# Debugging switch:
$DEBUG = "0";
if ($DEBUG) {
    use Sys::Syslog qw( :DEFAULT setlogsock);
}

use CCE;
use Sauce::Service;

my $cce = new CCE;
$cce->connectfd();

# Get Event Object:
my $obj = $cce->event_object();

# If the Event is CREATE or MODIFY and Vsite->{force_update} isn't yet set, then we do NOT
# restart Apache. On CREATE this is empty until the GUI has populated CODB with all options.
# Only then, as last act it sets Vsite->{force_update}. That is when we want to restart during
# a CREATE transaction. During MODIFY this will already be set from the CREATE *or* the last 
# MODIFY transaction. Hence during MODIFY we will always restart Apache. But only once, as there
# is only Handler that runs that matters and all Parameters that trigger *this* handler are in
# the same CLEANUP stage anyway.
if ((($cce->event_is_create()) || ($cce->event_is_modify())) && ($obj->{force_update} eq '')) {
    &debug_msg("Early exit: Doing nothing\n");
	$cce->bye('SUCCESS');
	exit(0);
}

#
### Static defines for PHP handling:
#

# Extra PHP location:
my $extra_PHP_basepath = '/home/solarspeed/';

#
### PHP Version related static defines:
#

# Known PHP versions:
%known_php_versions = (
                        'PHP53' => '5.3',
                        'PHP54' => '5.4',
                        'PHP55' => '5.5',
                        'PHP56' => '5.6',
                        'PHP70' => '7.0',
                        'PHP71' => '7.1',
                        'PHP72' => '7.2',
                        'PHP73' => '7.3',
                        'PHP74' => '7.4',
                        'PHP80' => '8.0',
                        'PHP81' => '8.1',
                        'PHP82' => '8.2',
                        'PHP83' => '8.3',
                        'PHP84' => '8.4',
                        'PHP85' => '8.5',
                        'PHP86' => '8.6',
                        'PHP90' => '9.0',
                        'PHP91' => '9.1',
                        'PHP92' => '9.2',
                        'PHP93' => '9.3',
                        'PHP94' => '9.4',
                        );

                        #'PHPOS' => '',

# Known PHP Services:
%known_php_services = (
                        'PHP53' => 'php-fpm-5.3',
                        'PHP54' => 'php-fpm-5.4',
                        'PHP55' => 'php-fpm-5.5',
                        'PHP56' => 'php-fpm-5.6',
                        'PHP70' => 'php-fpm-7.0',
                        'PHP71' => 'php-fpm-7.1',
                        'PHP72' => 'php-fpm-7.2',
                        'PHP73' => 'php-fpm-7.3',
                        'PHP74' => 'php-fpm-7.4',
                        'PHP80' => 'php-fpm-8.0',
                        'PHP81' => 'php-fpm-8.1',
                        'PHP82' => 'php-fpm-8.2',
                        'PHP83' => 'php-fpm-8.3',
                        'PHP84' => 'php-fpm-8.4',
                        'PHP85' => 'php-fpm-8.5',
                        'PHP86' => 'php-fpm-8.6',
                        'PHP90' => 'php-fpm-9.0',
                        'PHP91' => 'php-fpm-9.1',
                        'PHP92' => 'php-fpm-9.2',
                        'PHP93' => 'php-fpm-9.3',
                        'PHP94' => 'php-fpm-9.4',
                        );

                        #'PHPOS' => 'php-fpm',

#
### Handle PHP-FPM:
#

# This checks all known PHP-FPM versions if they are present and enabled. If a PHP-FPM
# version is present and enabled, it will be stopped. 

@fpm_need_to_restart = ();
for my $fpmKey (keys %known_php_versions) {

    $extra_dir = $extra_PHP_basepath . 'php-' . $known_php_versions{$fpmKey};
    if ((-d $extra_dir) || ($fpmKey eq 'PHPOS')) {

        $pidfile = '/run/php-fpm/' . $known_php_services{$fpmKey} . '.pid';
        if (-f $pidfile) {
            $fpm_pid = `cat $pidfile`;
            chomp($fpm_pid);
            &debug_msg("Runstate of $known_php_services{$fpmKey} is: running - sending HUP\n");
            `kill -HUP $fpm_pid`;
        }

        # Find out if that FPM service is enabled:
        my $fpmState = Sauce::Service::service_get_init($known_php_services{$fpmKey});

        $is_enabled = 'disabled';
        if ($fpmState == '1') {
            $is_enabled = 'enabled';
            # If it is enabled, we add it to the list of FPM servers we need to restart:
            push(@fpm_need_to_restart, $known_php_services{$fpmKey});
        }
        &debug_msg("Runstate of $known_php_services{$fpmKey} is: $is_enabled\n");
    }
}

#
### Restart all PHP-FPM versions selectively which are present and were enabled:
#

&debug_msg("Doing restart of php-fpm\n");
system("/usr/bin/systemctl reset-failed php-fpm");
Sauce::Service::service_run_init('php-fpm', 'restart');

foreach my $fpm_service (@fpm_need_to_restart) {
    &debug_msg("Doing restart of $fpm_service\n");
    system("/usr/bin/systemctl reset-failed $fpm_service");
    Sauce::Service::service_run_init($fpm_service, 'restart');
    sleep(1);
}

&debug_msg("Issuing service_run_init('httpd', 'reload') for event_object: " . $obj->{OID} . "");

@SysOID = $cce->find('System');
($ok, $Nginx) = $cce->get($SysOID[0], 'Nginx');

if ($Nginx->{enabled} == "1") {
    &debug_msg("Restarting Nginx and reloading Apache\n");
    $nginx_status = Sauce::Service::service_get_init("nginx");
    if ($nginx_status == "0") {        
        Sauce::Service::service_set_init('nginx', 'on');
    }
    Sauce::Service::service_run_init('httpd', 'reload');
    sleep(1);
    Sauce::Service::service_run_init('nginx', 'restart');
}
else {
    &debug_msg("Stopping and disabling Nginx, reloading Apache\n");
    $nginx_status = Sauce::Service::service_get_init("nginx");
    if ($nginx_status == "1") {        
        Sauce::Service::service_set_init('nginx', 'off');
    }
    Sauce::Service::service_run_init('nginx', 'stop');
    sleep(1);
    Sauce::Service::service_run_init('httpd', 'reload');
}

$cce->bye('SUCCESS');
exit(0);

# For debugging:
sub debug_msg {
    if ($DEBUG) {
        my $msg = shift;
        setlogsock('unix');
        openlog($0,'','user');
        syslog('info', "$ARGV[0]: $msg");
        closelog;
    }
}

# 
# Copyright (c) 2008-2024 Michael Stauber, SOLARSPEED.NET
# Copyright (c) 2008-2024 Team BlueOnyx, BLUEONYX.IT
# All Rights Reserved.
# 
# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its 
#    contributors may be used to endorse or promote products derived 
#    from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 
# You acknowledge that this software is not designed or intended for 
# use in the design, construction, operation or maintenance of any 
# nuclear facility.
# 